@@ -3,7 +3,7 @@ use inherit;
 $der = new inherit::CDerived();
 $str = $der->Foo();
 
-if ($str != "CBase::Foo") {
+if ($str ne "CBase::Foo") {
   die "test failed";
 }
 
