@@ -7,7 +7,7 @@
  * CHICKEN language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_chicken_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/chicken.cxx,v 1.57 2006/11/01 23:54:50 wsfulton Exp $";
+char cvsroot_chicken_cxx[] = "$Id: chicken.cxx 10270 2008-02-27 15:29:55Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -501,6 +501,8 @@ int CHICKEN::functionWrapper(Node *n) {
     }
   }
 
+  Setattr(n, "wrap:name", wname);
+
   /* Emit the function call */
   emit_action(n, f);
 
@@ -603,8 +605,6 @@ int CHICKEN::functionWrapper(Node *n) {
   Printv(f_wrappers, "static ", declfunc, " C_noret;\n", NIL);
   Wrapper_print(f, f_wrappers);
 
-  Setattr(n, "wrap:name", wname);
-
   /* Now register the function with the interpreter.   */
   if (!Getattr(n, "sym:overloaded")) {
     if (exporting_destructor && !no_collection) {
@@ -709,6 +709,8 @@ int CHICKEN::variableWrapper(Node *n) {
   if (overname) {
     Append(wname, overname);
   }
+  Setattr(n, "wrap:name", wname);
+
   // Check for interrupts
   Printv(f->code, "C_trace(\"", scmname, "\");\n", NIL);
 
@@ -876,18 +878,18 @@ int CHICKEN::constantWrapper(Node *n) {
 
   /* Special hook for member pointer */
   if (SwigType_type(t) == T_MPOINTER) {
-    Printf(f_header, "static %s = %s;\n", SwigType_str(t, wname), rvalue);
-    value = wname;
-  }
-  if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
-    Replaceall(tm, "$source", rvalue);
-    Replaceall(tm, "$target", source);
-    Replaceall(tm, "$result", source);
-    Replaceall(tm, "$value", rvalue);
-    Printf(f_header, "%s\n", tm);
+    Printf(f_header, "static %s = %s;\n", SwigType_str(t, source), rvalue);
   } else {
-    Swig_warning(WARN_TYPEMAP_CONST_UNDEF, input_file, line_number, "Unsupported constant value.\n");
-    return SWIG_NOWRAP;
+    if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
+      Replaceall(tm, "$source", rvalue);
+      Replaceall(tm, "$target", source);
+      Replaceall(tm, "$result", source);
+      Replaceall(tm, "$value", rvalue);
+      Printf(f_header, "%s\n", tm);
+    } else {
+      Swig_warning(WARN_TYPEMAP_CONST_UNDEF, input_file, line_number, "Unsupported constant value.\n");
+      return SWIG_NOWRAP;
+    }
   }
 
   f = NewWrapper();
@@ -907,6 +909,7 @@ int CHICKEN::constantWrapper(Node *n) {
 
   if (1 || (SwigType_type(t) != T_USER) || (isPointer(t))) {
 
+    Setattr(n, "wrap:name", wname);
     Printv(f->def, "static ", "void ", wname, "(C_word, C_word, C_word) C_noret;\n", NIL);
 
     Printv(f->def, "static ", "void ", wname, "(C_word argc, C_word closure, " "C_word continuation) {\n", NIL);
@@ -1162,6 +1165,8 @@ int CHICKEN::membervariableHandler(Node *n) {
   }
 
   Delete(proc);
+  Delete(setfunc);
+  Delete(getfunc);
   return SWIG_OK;
 }
 
