@@ -7,7 +7,7 @@
  * Guile language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_guile_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.44 2006/11/01 23:54:51 wsfulton Exp $";
+char cvsroot_guile_cxx[] = "$Id: guile.cxx 10270 2008-02-27 15:29:55Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -672,8 +672,10 @@ public:
       overname = Getattr(n, "sym:overname");
       args_passed_as_array = 1;
     } else {
-      if (!addSymbol(iname, n))
-	return SWIG_ERROR;
+      if (!addSymbol(iname, n)) {
+        DelWrapper(f);
+	return SWIG_ERROR; 
+      }
     }
     if (overname) {
       Append(wname, overname);
@@ -951,7 +953,7 @@ public:
 	if (use_scm_interface) {
 	  Printf(f_init, "scm_c_define_gsubr(\"%s\", 0, 0, 1, (swig_guile_proc) %s_rest);\n", proc_name, wname);
 	} else {
-	  Printf(f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s_rest, 0, 0, 1);\n", proc_name, wname, numreq, numargs - numreq);
+	  Printf(f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s_rest, 0, 0, 1);\n", proc_name, wname);
 	}
       } else if (emit_setters && struct_member && strlen(Char(proc_name)) > 3) {
 	int len = Len(proc_name);
@@ -1160,6 +1162,7 @@ public:
     // Build the name for scheme.
     proc_name = NewString(iname);
     Replaceall(proc_name, "_", "-");
+    Setattr(n, "wrap:name", proc_name);
 
     if (1 || (SwigType_type(t) != T_USER) || (is_a_pointer(t))) {
 
@@ -1371,6 +1374,7 @@ public:
     if ((SwigType_type(nctype) == T_USER) && (!is_a_pointer(nctype))) {
       Swig_warning(WARN_TYPEMAP_CONST_UNDEF, input_file, line_number, "Unsupported constant value.\n");
       Delete(var_name);
+      DelWrapper(f);
       return SWIG_NOWRAP;
     }
     // See if there's a typemap
