@@ -7,7 +7,7 @@
  * Main entry point to the SWIG core.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_main_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/main.cxx,v 1.93 2006/11/15 23:45:47 wsfulton Exp $";
+char cvsroot_main_cxx[] = "$Id: main.cxx 10348 2008-04-05 19:55:31Z wsfulton $";
 
 #include "swigconfig.h"
 
@@ -21,6 +21,7 @@ char cvsroot_main_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/main.cxx,v
 #include "swigwarn.h"
 #include "cparse.h"
 #include <ctype.h>
+#include <limits.h>		// for INT_MAX
 
 // Global variables
 
@@ -34,12 +35,23 @@ int AddExtern = 0;
 int NoExcept = 0;
 int SwigRuntime = 0;		// 0 = no option, 1 = -c or -runtime, 2 = -noruntime
 
+/* Suppress warning messages for private inheritance, preprocessor evaluation etc...
+   WARN_PP_EVALUATION            202
+   WARN_PARSE_PRIVATE_INHERIT    309
+   WARN_TYPE_ABSTRACT            403
+   WARN_LANG_OVERLOAD_CONST      512
+   WARN_PARSE_BUILTIN_NAME       321
+   WARN_PARSE_REDUNDANT          322
+ */
+#define EXTRA_WARNINGS "202,309,403,512,321,322"
 
 extern "C" {
   extern String *ModuleName;
 }
 
 /* usage string split into multiple parts otherwise string is too big for some compilers */
+/* naming conventions for commandline options - no underscores, no capital letters, join words together
+ * except when using a common prefix, then use '-' to separate, eg the debug-xxx options */
 static const char *usage1 = (const char *) "\
 \nGeneral Options\n\
      -addextern      - Add extra extern declarations\n\
@@ -48,18 +60,16 @@ static const char *usage1 = (const char *) "\
      -copyctor       - Automatically generate copy constructors wherever possible\n\
      -cpperraswarn   - Treat the preprocessor #error statement as #warning (default)\n\
      -copyright      - Display copyright notices\n\
-     -debug_template - Display information for debugging templates\n\
-     -debug_typemap  - Display information for debugging typemaps\n\
+     -debug-classes  - Display information about the classes found in the interface\n\
+     -debug-module <n>- Display module parse tree at stages 1-4, <n> is a csv list of stages\n\
+     -debug-tags     - Display information about the tags found in the interface\n\
+     -debug-template - Display information for debugging templates\n\
+     -debug-top <n>  - Display entire parse tree at stages 1-4, <n> is a csv list of stages\n\
+     -debug-typedef  - Display information about the types and typedefs in the interface\n\
+     -debug-typemap  - Display information for debugging typemaps\n\
      -directors      - Turn on director mode for all the classes, mainly for testing\n\
      -dirprot        - Turn on wrapping of protected members for director classes (default)\n\
      -D<symbol>      - Define a symbol <symbol> (for conditional compilation)\n\
-     -dump_classes   - Display information about the classes found in the interface\n\
-     -dump_module    - Display information on the module node tree avoiding system nodes\n\
-     -dump_parse_module - Display information on the module node tree after parsing avoiding system nodes\n\
-     -dump_parse_top - Display information on the node tree after parsing including system nodes\n\
-     -dump_tags      - Display information about the tags found in the interface\n\
-     -dump_top       - Display information on the entire node tree including system nodes\n\
-     -dump_typedef   - Display information about the types and typedefs in the interface\n\
      -E              - Preprocess only, does not generate wrapper code\n\
      -external-runtime [file] - Export the SWIG runtime stack\n\
      -fakeversion <v>- Make SWIG fake the program version number to <v>\n\
@@ -81,6 +91,7 @@ static const char *usage2 = (const char *) "\
      -importall      - Follow all #include statements as imports\n\
      -includeall     - Follow all #include statements\n\
      -l<ifile>       - Include SWIG library file <ifile>\n\
+     -macroerrors    - Report errors inside macros\n\
      -makedefault    - Create default constructors/destructors (the default)\n\
      -M              - List all dependencies\n\
      -MD             - Is equivalent to `-M -MF <file>', except `-E' is not implied\n\
@@ -101,7 +112,7 @@ static const char *usage2 = (const char *) "\
 
 static const char *usage3 = (const char *) "\
      -notemplatereduce - Disable reduction of the typedefs in templates\n\
-     -O              - Enable the optimizations options: \n\
+     -O              - Enable the optimization options: \n\
                         -fastdispatch -fvirtual \n\
      -o <outfile>    - Set name of the output file to <outfile>\n\
      -oh <headfile>  - Set name of the output header file to <headfile>\n\
@@ -111,16 +122,18 @@ static const char *usage3 = (const char *) "\
      -templatereduce - Reduce all the typedefs in templates\n\
      -v              - Run in verbose mode\n\
      -version        - Display SWIG version number\n\
-     -Wall           - Enable all warning messages\n\
+     -Wall           - Remove all warning suppression, also implies -Wextra\n\
      -Wallkw         - Enable keyword warnings for all the supported languages\n\
      -Werror         - Treat warnings as errors\n\
+     -Wextra         - Adds the following additional warnings: " EXTRA_WARNINGS "\n\
      -w<list>        - Suppress/add warning messages, eg -w401,+321 - see Warnings.html\n\
      -xmlout <file>  - Write XML version of the parse tree to <file> after normal processing\n\
 \n\
 Options can also be defined using the SWIG_FEATURES environment variable, for example:\n\
 \n\
   $ SWIG_FEATURES=\"-Wall\"\n\
-  $ swig -python interface.i \n\
+  $ export SWIG_FEATURES\n\
+  $ swig -python interface.i\n\
 \n\
 is equivalent to: \n\
 \n\
@@ -147,10 +160,8 @@ static char *outfile_name = 0;
 static char *outfile_name_h = 0;
 static int tm_debug = 0;
 static int dump_tags = 0;
-static int dump_top = 0;
 static int dump_module = 0;
-static int dump_parse_module = 0;
-static int dump_parse_top = 0;
+static int dump_top = 0;
 static int dump_xml = 0;
 static int browse = 0;
 static int dump_typedef = 0;
@@ -166,6 +177,7 @@ static String *dependencies_file = 0;
 static File *f_dependencies_file = 0;
 static int external_runtime = 0;
 static String *external_runtime_name = 0;
+enum { STAGE1=1, STAGE2=2, STAGE3=4, STAGE4=8, STAGEOVERFLOW=16 };
 
 // -----------------------------------------------------------------------------
 // check_suffix(char *name)
@@ -173,8 +185,8 @@ static String *external_runtime_name = 0;
 // Checks the suffix of a file to see if we should emit extern declarations.
 // -----------------------------------------------------------------------------
 
-static int check_suffix(char *name) {
-  char *c;
+static int check_suffix(const char *name) {
+  const char *c;
   if (!name)
     return 0;
   c = Swig_file_suffix(name);
@@ -224,6 +236,30 @@ static void install_opts(int argc, char *argv[]) {
   }
 }
 
+// -----------------------------------------------------------------------------
+// decode_numbers_list(String *numlist)
+// Decode comma separated list into a binary number of the inputs or'd together
+// eg list="1,4" will return (2^0 || 2^3) = 0x1001
+// ----------------------------------------------------------------------------- 
+
+static unsigned int decode_numbers_list(String *numlist) {
+  unsigned int decoded_number = 0;
+  if (numlist) {
+    List *numbers = Split(numlist, ',', INT_MAX);
+    if (numbers && Len(numbers) > 0) {
+      for (Iterator it = First(numbers); it.item; it = Next(it)) {
+        String *numstring = it.item;
+        // TODO: check that it is a number
+        int number = atoi(Char(numstring));
+        if (number > 0 && number <= 16) {
+          decoded_number |= (1 << (number-1));
+        }
+      }
+    }
+  }
+  return decoded_number;
+}
+
 // -----------------------------------------------------------------------------
 // Sets the output directory for language specific (proxy) files if not set and 
 // adds trailing file separator if necessary.
@@ -234,8 +270,8 @@ static void set_outdir(const String *c_wrapper_file_dir) {
   // Add file delimiter if not present in output directory name
   if (outdir && Len(outdir) != 0) {
     const char *outd = Char(outdir);
-    if (strcmp(outd + strlen(outd) - strlen(SWIG_FILE_DELIMETER), SWIG_FILE_DELIMETER) != 0)
-      Printv(outdir, SWIG_FILE_DELIMETER, NIL);
+    if (strcmp(outd + strlen(outd) - strlen(SWIG_FILE_DELIMITER), SWIG_FILE_DELIMITER) != 0)
+      Printv(outdir, SWIG_FILE_DELIMITER, NIL);
   }
   // Use the C wrapper file's directory if the output directory has not been set by user
   if (!outdir)
@@ -396,7 +432,9 @@ void SWIG_getoptions(int argc, char *argv[]) {
 	Swig_mark_arg(i);
       } else if (strncmp(argv[i], "-I", 2) == 0) {
 	// Add a new directory search path
-	Swig_add_directory((DOH *) Swig_copy_string(argv[i] + 2));
+	char *a = Swig_copy_string(argv[i] + 2);
+	Swig_add_directory((DOH *) a);
+	free(a);
 	Swig_mark_arg(i);
       } else if (strncmp(argv[i], "-D", 2) == 0) {
 	String *d = NewString(argv[i] + 2);
@@ -493,7 +531,7 @@ void SWIG_getoptions(int argc, char *argv[]) {
       } else if (strcmp(argv[i], "-addextern") == 0) {
 	AddExtern = 1;
 	Swig_mark_arg(i);
-      } else if ((strcmp(argv[i], "-show_templates") == 0) || (strcmp(argv[i], "-debug_template") == 0)) {
+      } else if ((strcmp(argv[i], "-debug-template") == 0) || (strcmp(argv[i], "-debug_template") == 0) || (strcmp(argv[i], "-show_templates") == 0)) {
 	Swig_cparse_debug_templates(1);
 	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-templatereduce") == 0) {
@@ -502,6 +540,9 @@ void SWIG_getoptions(int argc, char *argv[]) {
       } else if (strcmp(argv[i], "-notemplatereduce") == 0) {
 	SWIG_cparse_template_reduce(0);
 	Swig_mark_arg(i);
+      } else if (strcmp(argv[i], "-macroerrors") == 0) {
+	Swig_cparse_follow_locators(1);
+	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-swiglib") == 0) {
 	if (SwigLibWin)
 	  printf("%s\n", Char(SwigLibWin));
@@ -593,7 +634,7 @@ void SWIG_getoptions(int argc, char *argv[]) {
       } else if (strcmp(argv[i], "-nocpperraswarn") == 0) {
 	Preprocessor_error_as_warning(0);
 	Swig_mark_arg(i);
-      } else if ((strcmp(argv[i], "-tm_debug") == 0) || (strcmp(argv[i], "-debug_typemap") == 0)) {
+      } else if ((strcmp(argv[i], "-debug-typemap") == 0) || (strcmp(argv[i], "-debug_typemap") == 0) || (strcmp(argv[i], "-tm_debug") == 0)) {
 	tm_debug = 1;
 	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-module") == 0) {
@@ -643,23 +684,52 @@ void SWIG_getoptions(int argc, char *argv[]) {
       } else if (strcmp(argv[i], "-Werror") == 0) {
 	werror = 1;
 	Swig_mark_arg(i);
+      } else if (strcmp(argv[i], "-Wextra") == 0) {
+	Swig_mark_arg(i);
+        Swig_warnfilter(EXTRA_WARNINGS, 0);
       } else if (strncmp(argv[i], "-w", 2) == 0) {
 	Swig_mark_arg(i);
 	Swig_warnfilter(argv[i] + 2, 1);
-      } else if (strcmp(argv[i], "-dump_tags") == 0) {
+      } else if ((strcmp(argv[i], "-debug-tags") == 0) || (strcmp(argv[i], "-dump_tags") == 0)) {
 	dump_tags = 1;
 	Swig_mark_arg(i);
+      } else if (strcmp(argv[i], "-debug-top") == 0) {
+	Swig_mark_arg(i);
+	if (argv[i + 1]) {
+	  String *dump_list = NewString(argv[i + 1]);
+	  dump_top = decode_numbers_list(dump_list);
+          if (dump_top < STAGE1 || dump_top >= STAGEOVERFLOW)
+            Swig_arg_error();
+          else
+            Swig_mark_arg(i + 1);
+          Delete(dump_list);
+	} else {
+	  Swig_arg_error();
+	}
+      } else if (strcmp(argv[i], "-debug-module") == 0) {
+	Swig_mark_arg(i);
+	if (argv[i + 1]) {
+	  String *dump_list = NewString(argv[i + 1]);
+	  dump_module = decode_numbers_list(dump_list);
+          if (dump_module < STAGE1 || dump_module >= STAGEOVERFLOW)
+            Swig_arg_error();
+          else
+            Swig_mark_arg(i + 1);
+          Delete(dump_list);
+	} else {
+	  Swig_arg_error();
+	}
       } else if ((strcmp(argv[i], "-dump_tree") == 0) || (strcmp(argv[i], "-dump_top") == 0)) {
-	dump_top = 1;
+	dump_top |= STAGE4;
 	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-dump_module") == 0) {
-	dump_module = 1;
+	dump_module |= STAGE4;
 	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-dump_parse_module") == 0) {
-	dump_parse_module = 1;
+	dump_module |= STAGE1;
 	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-dump_parse_top") == 0) {
-	dump_parse_top = 1;
+	dump_top |= STAGE1;
 	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-dump_xml") == 0) {
 	dump_xml = 1;
@@ -679,13 +749,13 @@ void SWIG_getoptions(int argc, char *argv[]) {
       } else if (strcmp(argv[i], "-browse") == 0) {
 	browse = 1;
 	Swig_mark_arg(i);
-      } else if (strcmp(argv[i], "-dump_typedef") == 0) {
+      } else if ((strcmp(argv[i], "-debug-typedef") == 0) || (strcmp(argv[i], "-dump_typedef") == 0)) {
 	dump_typedef = 1;
 	Swig_mark_arg(i);
-      } else if (strcmp(argv[i], "-dump_classes") == 0) {
+      } else if ((strcmp(argv[i], "-debug-classes") == 0) || (strcmp(argv[i], "-dump_classes") == 0)) {
 	dump_classes = 1;
 	Swig_mark_arg(i);
-      } else if (strcmp(argv[i], "-dump_memory") == 0) {
+      } else if ((strcmp(argv[i], "-debug-memory") == 0) || (strcmp(argv[i], "-dump_memory") == 0)) {
 	memory_debug = 1;
 	Swig_mark_arg(i);
       } else if (strcmp(argv[i], "-Fstandard") == 0) {
@@ -715,21 +785,13 @@ void SWIG_getoptions(int argc, char *argv[]) {
 
 int SWIG_main(int argc, char *argv[], Language *l) {
   char *c;
+  extern void Swig_print_xml(Node *obj, String *filename);
 
   /* Initialize the SWIG core */
   Swig_init();
 
-  /* Suppress warning messages for private inheritance, preprocessor
-     evaluation, might be abstract, overloaded const, and ...
-
-     WARN_PP_EVALUATION            202
-     WARN_PARSE_PRIVATE_INHERIT    309
-     WARN_TYPE_ABSTRACT            403
-     WARN_LANG_OVERLOAD_CONST      512
-     WARN_PARSE_BUILTIN_NAME       321
-     WARN_PARSE_REDUNDANT          322
-   */
-  Swig_warnfilter("202,309,403,512,321,322", 1);
+  // Default warning suppression
+  Swig_warnfilter(EXTRA_WARNINGS, 1);
 
   // Initialize the preprocessor
   Preprocessor_init();
@@ -817,19 +879,19 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   // Add language dependent directory to the search path
   {
     String *rl = NewString("");
-    Printf(rl, ".%sswig_lib%s%s", SWIG_FILE_DELIMETER, SWIG_FILE_DELIMETER, LangSubDir);
+    Printf(rl, ".%sswig_lib%s%s", SWIG_FILE_DELIMITER, SWIG_FILE_DELIMITER, LangSubDir);
     Swig_add_directory(rl);
     if (SwigLibWin) {
       rl = NewString("");
-      Printf(rl, "%s%s%s", SwigLibWin, SWIG_FILE_DELIMETER, LangSubDir);
+      Printf(rl, "%s%s%s", SwigLibWin, SWIG_FILE_DELIMITER, LangSubDir);
       Swig_add_directory(rl);
     }
     rl = NewString("");
-    Printf(rl, "%s%s%s", SwigLib, SWIG_FILE_DELIMETER, LangSubDir);
+    Printf(rl, "%s%s%s", SwigLib, SWIG_FILE_DELIMITER, LangSubDir);
     Swig_add_directory(rl);
   }
 
-  Swig_add_directory((String *) "." SWIG_FILE_DELIMETER "swig_lib");
+  Swig_add_directory((String *) "." SWIG_FILE_DELIMITER "swig_lib");
   if (SwigLibWin)
     Swig_add_directory((String *) SwigLibWin);
   Swig_add_directory((String *) SwigLib);
@@ -890,6 +952,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
     // Run the preprocessor
     if (Verbose)
       printf("Preprocessing...\n");
+
     {
       int i;
       String *fs = NewString("");
@@ -898,7 +961,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	char *cfile = Char(input_file);
 	if (cfile && cfile[0] == '-') {
 	  Printf(stderr, "Unable to find option or file '%s', ", input_file);
-	  Printf(stderr, "use 'swig -help' for more information.\n", input_file);
+	  Printf(stderr, "use 'swig -help' for more information.\n");
 	} else {
 	  Printf(stderr, "Unable to find file '%s'.\n", input_file);
 	}
@@ -929,13 +992,12 @@ int SWIG_main(int argc, char *argv[], Language *l) {
       }
       if (cpp_only) {
 	Printf(stdout, "%s", cpps);
-	while (freeze);
 	SWIG_exit(EXIT_SUCCESS);
       }
       if (depend) {
 	String *outfile;
 	if (!outfile_name) {
-	  if (CPlusPlus) {
+	  if (CPlusPlus || lang->cplus_runtime_mode()) {
 	    outfile = NewStringf("%s_wrap.%s", Swig_file_basename(input_file), cpp_extension);
 	  } else {
 	    outfile = NewStringf("%s_wrap.c", Swig_file_basename(input_file));
@@ -985,10 +1047,12 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 
     Node *top = Swig_cparse(cpps);
 
-    if (dump_parse_top) {
+    if (dump_top & STAGE1) {
+      Printf(stdout, "debug-top stage 1\n");
       Swig_print_tree(top);
     }
-    if (dump_parse_module) {
+    if (dump_module & STAGE1) {
+      Printf(stdout, "debug-module stage 1\n");
       Swig_print_tree(Getattr(top, "module"));
     }
 
@@ -997,11 +1061,29 @@ int SWIG_main(int argc, char *argv[], Language *l) {
     }
     Swig_process_types(top);
 
+    if (dump_top & STAGE2) {
+      Printf(stdout, "debug-top stage 2\n");
+      Swig_print_tree(top);
+    }
+    if (dump_module & STAGE2) {
+      Printf(stdout, "debug-module stage 2\n");
+      Swig_print_tree(Getattr(top, "module"));
+    }
+
     if (Verbose) {
       Printf(stdout, "C++ analysis...\n");
     }
     Swig_default_allocators(top);
 
+    if (dump_top & STAGE3) {
+      Printf(stdout, "debug-top stage 3\n");
+      Swig_print_tree(top);
+    }
+    if (dump_module & STAGE3) {
+      Printf(stdout, "debug-module stage 3\n");
+      Swig_print_tree(Getattr(top, "module"));
+    }
+
     if (Verbose) {
       Printf(stdout, "Generating wrappers...\n");
     }
@@ -1033,7 +1115,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	/* Set some filename information on the object */
 	Setattr(top, "infile", input_file);
 	if (!outfile_name) {
-	  if (CPlusPlus) {
+	  if (CPlusPlus || lang->cplus_runtime_mode()) {
 	    Setattr(top, "outfile", NewStringf("%s_wrap.%s", Swig_file_basename(input_file), cpp_extension));
 	  } else {
 	    Setattr(top, "outfile", NewStringf("%s_wrap.c", Swig_file_basename(input_file)));
@@ -1056,13 +1138,15 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	}
       }
     }
-    if (dump_top) {
+    if (dump_top & STAGE4) {
+      Printf(stdout, "debug-top stage 4\n");
       Swig_print_tree(top);
     }
-    if (dump_module) {
+    if (dump_module & STAGE4) {
+      Printf(stdout, "debug-module stage 4\n");
       Swig_print_tree(Getattr(top, "module"));
     }
-    if (dump_xml) {
+    if (dump_xml && top) {
       Swig_print_xml(top, xmlout);
     }
     Delete(top);
@@ -1076,8 +1160,8 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   Delete(libfiles);
   Preprocessor_delete();
 
-  while (freeze);
-
+  while (freeze) {
+  }
 
   if ((werror) && (Swig_warn_count())) {
     return Swig_warn_count();
@@ -1092,6 +1176,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 // --------------------------------------------------------------------------
 
 void SWIG_exit(int exit_code) {
-  while (freeze);
+  while (freeze) {
+  }
   exit(exit_code);
 }
