@@ -7,7 +7,7 @@
  * Ocaml language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx,v 1.78 2006/11/09 22:32:28 wsfulton Exp $";
+char cvsroot_ocaml_cxx[] = "$Id: ocaml.cxx 10270 2008-02-27 15:29:55Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -75,7 +75,7 @@ public:
   }
 
   void PrintIncludeArg() {
-    Printv(stdout, SWIG_LIB, SWIG_FILE_DELIMETER, ocaml_path, "\n", NIL);
+    Printv(stdout, SWIG_LIB, SWIG_FILE_DELIMITER, ocaml_path, "\n", NIL);
   }
 
   /* ------------------------------------------------------------
@@ -311,7 +311,7 @@ public:
     Printf(f_int_to_enum, "let _ = Callback.register \"%s_int_to_enum\" int_to_enum\n", module);
     Printf(f_mlibody, "val int_to_enum : c_enum_type -> int -> c_obj\n");
     Printf(f_init, "#define SWIG_init f_%s_init\n" "%s" "}\n", module, init_func_def);
-    Printf(f_mlbody, "external f_init : unit -> unit = \"f_%s_init\" ;;\n" "let _ = f_init ()\n", module, module);
+    Printf(f_mlbody, "external f_init : unit -> unit = \"f_%s_init\" ;;\n" "let _ = f_init ()\n", module);
     Printf(f_enumtypes_type, "]\n");
     Printf(f_enumtypes_value, "]\n\n" "type c_obj = c_enum_value c_obj_t\n");
 
@@ -446,8 +446,10 @@ public:
     if (isOverloaded) {
       overname = Getattr(n, "sym:overname");
     } else {
-      if (!addSymbol(iname, n))
+      if (!addSymbol(iname, n)) {
+        DelWrapper(f);
 	return SWIG_ERROR;
+      }
     }
     if (overname) {
       Append(wname, overname);
@@ -469,7 +471,7 @@ public:
     } else if (classmode && in_destructor) {
       Printf(f_class_ctors, "    \"~\", %s ;\n", mangled_name);
     } else if (classmode && !in_constructor && !in_destructor && !static_member_function) {
-      String *opname = Copy(Getattr(n, "name"));
+      String *opname = Copy(Getattr(n, "memberfunctionHandler:sym:name"));
 
       Replaceall(opname, "operator ", "");
 
@@ -793,6 +795,7 @@ public:
 
     // Build the name for scheme.
     Printv(proc_name, iname, NIL);
+    Setattr(n, "wrap:name", proc_name);
 
     Printf(f->def, "SWIGEXT CAML_VALUE %s(CAML_VALUE args) {\n", var_name);
     // Printv(f->def, "#define FUNC_NAME \"", proc_name, "\"", NIL);
@@ -1094,6 +1097,10 @@ public:
 
   int classHandler(Node *n) {
     String *name = Getattr(n, "name");
+
+    if (!name)
+      return SWIG_OK;
+
     String *mangled_sym_name = mangleNameForCaml(name);
     String *this_class_def = NewString(f_classtemplate);
     String *name_normalized = normalizeTemplatedClassName(name);
@@ -1102,8 +1109,6 @@ public:
     f_class_ctors = NewString("");
     bool sizeof_feature = generate_sizeof && isSimpleType(name);
 
-    if (!name)
-      return SWIG_OK;
 
     classname = mangled_sym_name;
     classmode = true;
