@@ -13,13 +13,11 @@ char cvsroot_octave_cxx[] = "$Id$";
 
 static const char *usage = (char *) "\
 Octave Options (available with -octave)\n\
-     -api <N> - Generate code that assumes Octave API N [default: 37]\n\
-     \n";
+     (none yet)\n\n";
 
 
 class OCTAVE:public Language {
 private:
-  File *f_begin;
   File *f_runtime;
   File *f_header;
   File *f_doc;
@@ -36,27 +34,17 @@ private:
   int have_destructor;
   String *constructor_name;
 
-  int api_version;
-
   Hash *docs;
 
 public:
-   OCTAVE():f_begin(0), f_runtime(0), f_header(0), f_doc(0), f_wrappers(0),
+   OCTAVE():f_runtime(0), f_header(0), f_doc(0), f_wrappers(0),
 	    f_init(0), f_initbeforefunc(0), f_directors(0), f_directors_h(0), 
 	    s_global_tab(0), s_members_tab(0), class_name(0) {
-     /* Add code to manage protected constructors and directors */
-     director_prot_ctor_code = NewString("");
-     Printv(director_prot_ctor_code,
-	    "if ( $comparison ) { /* subclassed */\n",
-	    "  $director_new \n",
-	    "} else {\n", "  error(\"accessing abstract class or protected constructor\"); \n", "  SWIG_fail;\n", "}\n", NIL);
-
      enable_cplus_runtime_mode();
      allow_overloading();
      director_multiple_inheritance = 1;
      director_language = 1;
      docs = NewHash();
-     api_version = 0;
    }
 
   virtual void main(int argc, char *argv[]) {
@@ -64,15 +52,6 @@ public:
       if (argv[i]) {
 	if (strcmp(argv[i], "-help") == 0) {
 	  fputs(usage, stderr);
-	} else if (strcmp(argv[i], "-api") == 0) {
-	  if (argv[i + 1]) {
-	    api_version = atoi(argv[i + 1]);
-	    Swig_mark_arg(i);
-	    Swig_mark_arg(i + 1);
-	    i++;
-	  } else {
-	    Swig_arg_error();
-	  }
 	}
       }
     }
@@ -108,12 +87,11 @@ public:
 
     String *module = Getattr(n, "name");
     String *outfile = Getattr(n, "outfile");
-    f_begin = NewFile(outfile, "w", SWIG_output_files());
-    if (!f_begin) {
+    f_runtime = NewFile(outfile, "w");
+    if (!f_runtime) {
       FileErrorDisplay(outfile);
       SWIG_exit(EXIT_FAILURE);
     }
-    f_runtime = NewString("");
     f_header = NewString("");
     f_doc = NewString("");
     f_wrappers = NewString("");
@@ -122,7 +100,6 @@ public:
     f_directors_h = NewString("");
     f_directors = NewString("");
     s_global_tab = NewString("");
-    Swig_register_filebyname("begin", f_begin);
     Swig_register_filebyname("runtime", f_runtime);
     Swig_register_filebyname("header", f_header);
     Swig_register_filebyname("doc", f_doc);
@@ -131,17 +108,11 @@ public:
     Swig_register_filebyname("initbeforefunc", f_initbeforefunc);
     Swig_register_filebyname("director", f_directors);
     Swig_register_filebyname("director_h", f_directors_h);
-
-    Swig_banner(f_begin);
-
-    Printf(f_runtime, "\n");
-    Printf(f_runtime, "#define SWIGOCTAVE\n");
+    Swig_banner(f_runtime);
     Printf(f_runtime, "#define SWIG_name_d      \"%s\"\n", module);
     Printf(f_runtime, "#define SWIG_name        %s\n", module);
-    Printf(f_runtime, "#define OCTAVE_API_VERSION_OPTION %i\n", api_version);
 
     if (directorsEnabled()) {
-      Printf(f_runtime, "#define SWIG_DIRECTORS\n");
       Swig_banner(f_directors_h);
       if (dirprot_mode()) {
 	//      Printf(f_directors_h, "#include <map>\n");
@@ -149,10 +120,9 @@ public:
       }
     }
 
-    Printf(f_runtime, "\n");
 
     Printf(s_global_tab, "\nstatic const struct swig_octave_member swig_globals[] = {\n");
-    Printf(f_init, "static void SWIG_init_user(octave_swig_type* module_ns)\n{\n");
+    Printf(f_init, "void SWIG_init_user(octave_swig_type* module_ns)\n{\n");
 
     if (!CPlusPlus)
       Printf(f_header,"extern \"C\" {\n");
@@ -173,16 +143,15 @@ public:
 
     Printv(f_wrappers, s_global_tab, NIL);
     SwigType_emit_type_table(f_runtime, f_wrappers);
-    Dump(f_runtime, f_begin);
-    Dump(f_header, f_begin);
-    Dump(f_doc, f_begin);
+    Dump(f_header, f_runtime);
+    Dump(f_doc, f_runtime);
     if (directorsEnabled()) {
-      Dump(f_directors_h, f_begin);
-      Dump(f_directors, f_begin);
+      Dump(f_directors_h, f_runtime);
+      Dump(f_directors, f_runtime);
     }
-    Dump(f_wrappers, f_begin);
-    Dump(f_initbeforefunc, f_begin);
-    Wrapper_pretty_print(f_init, f_begin);
+    Dump(f_wrappers, f_runtime);
+    Dump(f_initbeforefunc, f_runtime);
+    Wrapper_pretty_print(f_init, f_runtime);
 
     Delete(s_global_tab);
     Delete(f_initbeforefunc);
@@ -192,9 +161,8 @@ public:
     Delete(f_header);
     Delete(f_directors);
     Delete(f_directors_h);
-    Close(f_begin);
+    Close(f_runtime);
     Delete(f_runtime);
-    Delete(f_begin);
 
     return SWIG_OK;
   }
@@ -458,7 +426,7 @@ public:
 
     Printv(f->def, "static octave_value_list ", overname, " (const octave_value_list& args, int nargout) {", NIL);
 
-    emit_parameter_variables(l, f);
+    emit_args(d, l, f);
     emit_attach_parmmaps(l, f);
     Setattr(n, "wrap:parms", l);
 
@@ -601,15 +569,14 @@ public:
 
     Setattr(n, "wrap:name", overname);
 
-    Swig_director_emit_dynamic_cast(n, f);
-    String *actioncode = emit_action(n);
+    emit_action(n, f);
 
     Wrapper_add_local(f, "_out", "octave_value_list _out");
     Wrapper_add_local(f, "_outp", "octave_value_list *_outp=&_out");
     Wrapper_add_local(f, "_outv", "octave_value _outv");
 
     // Return the function value
-    if ((tm = Swig_typemap_lookup_out("out", n, "result", f, actioncode))) {
+    if ((tm = Swig_typemap_lookup_new("out", n, "result", 0))) {
       Replaceall(tm, "$source", "result");
       Replaceall(tm, "$target", "_outv");
       Replaceall(tm, "$result", "_outv");
@@ -625,19 +592,18 @@ public:
     } else {
       Swig_warning(WARN_TYPEMAP_OUT_UNDEF, input_file, line_number, "Unable to use return type %s in function %s.\n", SwigType_str(d, 0), iname);
     }
-    emit_return_variable(n, d, f);
 
     Printv(f->code, outarg, NIL);
     Printv(f->code, cleanup, NIL);
 
     if (GetFlag(n, "feature:new")) {
-      if ((tm = Swig_typemap_lookup("newfree", n, "result", 0))) {
+      if ((tm = Swig_typemap_lookup_new("newfree", n, "result", 0))) {
 	Replaceall(tm, "$source", "result");
 	Printf(f->code, "%s\n", tm);
       }
     }
 
-    if ((tm = Swig_typemap_lookup("ret", n, "result", 0))) {
+    if ((tm = Swig_typemap_lookup_new("ret", n, "result", 0))) {
       Replaceall(tm, "$source", "result");
       Replaceall(tm, "$result", "_outv");
       Printf(f->code, "%s\n", tm);
@@ -717,14 +683,14 @@ public:
     Printf(setf->def, "if (!SWIG_check_num_args(\"%s_set\",args.length(),1,1,0)) return octave_value_list();", iname);
     if (is_assignable(n)) {
       Setattr(n, "wrap:name", setname);
-      if ((tm = Swig_typemap_lookup("varin", n, name, 0))) {
+      if ((tm = Swig_typemap_lookup_new("varin", n, name, 0))) {
 	Replaceall(tm, "$source", "args(0)");
 	Replaceall(tm, "$target", name);
 	Replaceall(tm, "$input", "args(0)");
 	if (Getattr(n, "tmap:varin:implicitconv")) {
 	  Replaceall(tm, "$implicitconv", get_implicitconv_flag(n));
 	}
-	emit_action_code(n, setf->code, tm);
+	emit_action_code(n, setf, tm);
 	Delete(tm);
       } else {
 	Swig_warning(WARN_TYPEMAP_VARIN_UNDEF, input_file, line_number, "Unable to set variable of type %s.\n", SwigType_str(t, 0));
@@ -741,11 +707,11 @@ public:
     int addfail = 0;
     Printf(getf->def, "static octave_value_list _wrap_%s(const octave_value_list& args,int nargout) {", getname);
     Wrapper_add_local(getf, "obj", "octave_value obj");
-    if ((tm = Swig_typemap_lookup("varout", n, name, 0))) {
+    if ((tm = Swig_typemap_lookup_new("varout", n, name, 0))) {
       Replaceall(tm, "$source", name);
       Replaceall(tm, "$target", "obj");
       Replaceall(tm, "$result", "obj");
-      addfail = emit_action_code(n, getf->code, tm);
+      addfail = emit_action_code(n, getf, tm);
       Delete(tm);
     } else {
       Swig_warning(WARN_TYPEMAP_VAROUT_UNDEF, input_file, line_number, "Unable to read variable of type %s\n", SwigType_str(t, 0));
@@ -781,7 +747,7 @@ public:
       Delete(str);
       value = wname;
     }
-    if ((tm = Swig_typemap_lookup("constcode", n, name, 0))) {
+    if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
       Replaceall(tm, "$source", value);
       Replaceall(tm, "$target", name);
       Replaceall(tm, "$value", value);
@@ -1317,8 +1283,13 @@ public:
 	Printf(w->code, "}\n");
 
 	Setattr(n, "type", return_type);
-	tm = Swig_typemap_lookup("directorout", n, "result", w);
+	tm = Swig_typemap_lookup_new("directorout", n, "result", w);
 	Setattr(n, "type", type);
+	if (tm == 0) {
+	  String *name = NewString("result");
+	  tm = Swig_typemap_search("directorout", return_type, name, NULL);
+	  Delete(name);
+	}
 	if (tm != 0) {
 	  char temp[24];
 	  sprintf(temp, "out(%d)", idx);
