@@ -9,18 +9,14 @@
  * are provided.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_include_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/include.c,v 1.33 2006/11/01 23:54:53 wsfulton Exp $";
+char cvsroot_include_c[] = "$Id: include.c 9967 2007-10-08 12:02:56Z olly $";
 
 #include "swig.h"
-#include "swigkeys.h"
 
-/* Delimeter used in accessing files and directories */
-
-static List *directories = 0;	/* List of include directories */
-static String *lastpath = 0;	/* Last file that was included */
-static String *swiglib = 0;	/* Location of SWIG library */
-static String *lang_config = 0;	/* Language configuration file */
-static int dopush = 1;		/* Whether to push directories */
+static List   *directories = 0;	        /* List of include directories */
+static String *lastpath = 0;	        /* Last file that was included */
+static List   *pdirectories = 0;        /* List of pushed directories  */
+static int     dopush = 1;		/* Whether to push directories */
 
 /* This functions determine whether to push/pop dirs in the preprocessor */
 void Swig_set_push_dir(int push) {
@@ -31,34 +27,6 @@ int Swig_get_push_dir(void) {
   return dopush;
 }
 
-
-/* This function sets the name of the configuration file */
-
-void Swig_set_config_file(const String_or_char *filename) {
-  lang_config = NewString(filename);
-}
-
-String *Swig_get_config_file() {
-  return lang_config;
-}
-
-
-/* -----------------------------------------------------------------------------
- * Swig_swiglib_set()
- * Swig_swiglib_get()
- *
- * Set the location of the SWIG library.  This isn't really used, by the
- * include mechanism, but rather as a query interface for language modules.
- * ----------------------------------------------------------------------------- */
-
-void Swig_swiglib_set(const String_or_char *sl) {
-  swiglib = NewString(sl);
-}
-
-String *Swig_swiglib_get() {
-  return swiglib;
-}
-
 /* -----------------------------------------------------------------------------
  * Swig_add_directory()
  *
@@ -66,24 +34,18 @@ String *Swig_swiglib_get() {
  * ----------------------------------------------------------------------------- */
 
 List *Swig_add_directory(const String_or_char *dirname) {
+  String *adirname;
   if (!directories)
     directories = NewList();
   assert(directories);
   if (dirname) {
-    String *sdir = NewString(dirname);
-    Hash *dir = NewHash();
-    assert(dir);
-    SetFlag(dir, k_sysdir);
-    Setattr(dir, k_name, sdir);
-    Append(directories, dir);
-    Delete(dir);
-    Delete(sdir);
+    adirname = NewString(dirname);
+    Append(directories,adirname);
+    Delete(adirname);
   }
   return directories;
 }
 
-
-
 /* -----------------------------------------------------------------------------
  * Swig_push_directory()
  *
@@ -92,23 +54,16 @@ List *Swig_add_directory(const String_or_char *dirname) {
  * ----------------------------------------------------------------------------- */
 
 void Swig_push_directory(const String_or_char *dirname) {
-  String *tmp = 0;
+  String *pdirname;
   if (!Swig_get_push_dir())
     return;
-  if (!directories)
-    directories = NewList();
-  assert(directories);
-  if (!DohIsString(dirname)) {
-    dirname = tmp = NewString(dirname);
-    assert(dirname);
-  }
-  if (dirname) {
-    Hash *dir = NewHash();
-    Setattr(dir, k_name, dirname);
-    Insert(directories, 0, dir);
-    if (tmp)
-      Delete(tmp);
-  }
+  if (!pdirectories)
+    pdirectories = NewList();
+  assert(pdirectories);
+  pdirname = NewString(dirname);
+  assert(pdirname);
+  Insert(pdirectories,0,pdirname);
+  Delete(pdirname);
 }
 
 /* -----------------------------------------------------------------------------
@@ -121,9 +76,9 @@ void Swig_push_directory(const String_or_char *dirname) {
 void Swig_pop_directory() {
   if (!Swig_get_push_dir())
     return;
-  if (!directories)
+  if (!pdirectories)
     return;
-  Delitem(directories, 0);
+  Delitem(pdirectories, 0);
 }
 
 /* -----------------------------------------------------------------------------
@@ -145,57 +100,45 @@ String *Swig_last_file() {
 
 static List *Swig_search_path_any(int syspath) {
   String *filename;
-  String *dirname;
-  List *slist, *llist;
-  int i, ilen;
+  List   *slist;
+  int     i, ilen;
 
-  llist = 0;
   slist = NewList();
   assert(slist);
   filename = NewStringEmpty();
   assert(filename);
 #ifdef MACSWIG
-  Printf(filename, "%s", SWIG_FILE_DELIMETER);
+  Printf(filename, "%s", SWIG_FILE_DELIMITER);
 #else
-  Printf(filename, ".%s", SWIG_FILE_DELIMETER);
+  Printf(filename, ".%s", SWIG_FILE_DELIMITER);
 #endif
-  if (syspath) {
-    llist = NewList();
-    assert(llist);
-    Append(llist, filename);
-  } else {
-    Append(slist, filename);
+  Append(slist, filename);
+  Delete(filename);
+  
+  /* If there are any pushed directories.  Add them first */
+  if (pdirectories) {
+    ilen = Len(pdirectories);
+    for (i = 0; i < ilen; i++) {
+      filename = NewString(Getitem(pdirectories,i));
+      Append(filename,SWIG_FILE_DELIMITER);
+      Append(slist,filename);
+      Delete(filename);
+    }
   }
+  /* Add system directories next */
   ilen = Len(directories);
   for (i = 0; i < ilen; i++) {
-    int issimple = 0;
-    dirname = Getitem(directories, i);
-    filename = NewStringEmpty();
-    assert(filename);
-    if (DohIsString(dirname)) {
-      filename = Copy(dirname);
-      issimple = 1;
+    filename = NewString(Getitem(directories,i));
+    Append(filename,SWIG_FILE_DELIMITER);
+    if (syspath) {
+      /* If doing a system include, put the system directories first */
+      Insert(slist,i,filename);
     } else {
-      filename = Copy(Getattr(dirname, k_name));
-    }
-    StringAppend(filename, SWIG_FILE_DELIMETER);
-
-    if (syspath && (issimple || !GetFlag(dirname, k_sysdir))) {
-      Append(llist, filename);
-    } else {
-      Append(slist, filename);
-      /* Insert(slist,0,filename); */
+      /* Otherwise, just put the system directories after the pushed directories (if any) */
+      Append(slist,filename);
     }
     Delete(filename);
   }
-  if (syspath) {
-    int ilen = Len(llist);
-    for (i = 0; i < ilen; i++) {
-      Append(slist, Getitem(llist, i));
-    }
-    Delete(llist);
-  }
-
   return slist;
 }
 
@@ -269,16 +212,15 @@ String *Swig_read_file(FILE *f) {
 
   assert(str);
   while (fgets(buffer, 4095, f)) {
-    StringAppend(str, buffer);
+    Append(str, buffer);
   }
-  len = StringLen(str);
+  len = Len(str);
   if (len) {
     char *cstr = Char(str);
     if (cstr[len - 1] != '\n') {
-      StringAppend(str, "\n");
+      Append(str, "\n");
     }
   }
-
   return str;
 }
 
@@ -314,7 +256,6 @@ String *Swig_include_sys(const String_or_char *name) {
   return Swig_include_any(name, 1);
 }
 
-
 /* -----------------------------------------------------------------------------
  * Swig_insert_file()
  *
@@ -407,7 +348,7 @@ char *Swig_file_basename(const String_or_char *filename) {
  * ----------------------------------------------------------------------------- */
 char *Swig_file_filename(const String_or_char *filename) {
   static char tmp[1024];
-  const char *delim = SWIG_FILE_DELIMETER;
+  const char *delim = SWIG_FILE_DELIMITER;
   char *c;
 
   strcpy(tmp, Char(filename));
@@ -424,7 +365,7 @@ char *Swig_file_filename(const String_or_char *filename) {
  * ----------------------------------------------------------------------------- */
 char *Swig_file_dirname(const String_or_char *filename) {
   static char tmp[1024];
-  const char *delim = SWIG_FILE_DELIMETER;
+  const char *delim = SWIG_FILE_DELIMITER;
   char *c;
   strcpy(tmp, Char(filename));
   if (!strstr(tmp, delim)) {
