@@ -7,7 +7,7 @@
  * Header file for the SWIG core.
  * ----------------------------------------------------------------------------- */
 
-/* $Header: /cvsroot/swig/SWIG/Source/Swig/swig.h,v 1.124 2006/11/15 23:45:47 wsfulton Exp $ */
+/* $Id: swig.h 10266 2008-02-27 13:01:17Z wsfulton $ */
 
 #ifndef SWIGCORE_H_
 #define SWIGCORE_H_
@@ -92,118 +92,18 @@ extern "C" {
 #define   T_SYMBOL     98
 #define   T_ERROR      99
 
+
 /* --- File interface --- */
 
-  extern List *Swig_add_directory(const String_or_char *dirname);
-  extern void Swig_push_directory(const String_or_char *dirname);
-  extern void Swig_pop_directory();
-  extern String *Swig_last_file();
-  extern List *Swig_search_path();
-  extern FILE *Swig_open(const String_or_char *name);
-  extern String *Swig_read_file(FILE *f);
-  extern String *Swig_include(const String_or_char *name);
-  extern String *Swig_include_sys(const String_or_char *name);
-  extern int Swig_insert_file(const String_or_char *name, File *outfile);
-  extern void Swig_set_config_file(const String_or_char *filename);
-  extern String *Swig_get_config_file(void);
-  extern void Swig_set_push_dir(int dopush);
-  extern int Swig_get_push_dir(void);
-  extern void Swig_swiglib_set(const String_or_char *);
-  extern String *Swig_swiglib_get();
-  extern void Swig_register_filebyname(const String_or_char *filename, File *outfile);
-  extern File *Swig_filebyname(const String_or_char *filename);
-  extern char *Swig_file_suffix(const String_or_char *filename);
-  extern char *Swig_file_basename(const String_or_char *filename);
-  extern char *Swig_file_filename(const String_or_char *filename);
-  extern char *Swig_file_dirname(const String_or_char *filename);
-
-#if defined(MACSWIG)
-#  define SWIG_FILE_DELIMETER  ":"
-#elif defined(_WIN32)
-#  define  SWIG_FILE_DELIMETER "\\"
-#else
-#  define  SWIG_FILE_DELIMETER "/"
-#endif
+#include "swigfile.h"
 
 /* --- Command line parsing --- */
 
-  extern void Swig_init_args(int argc, char **argv);
-  extern void Swig_mark_arg(int n);
-  extern int Swig_check_marked(int n);
-  extern void Swig_check_options(int check_input);
-  extern void Swig_arg_error();
+#include "swigopt.h"
 
 /* --- Scanner Interface --- */
 
-  typedef struct SwigScanner SwigScanner;
-
-  extern SwigScanner *NewSwigScanner();
-  extern void DelSwigScanner(SwigScanner *);
-  extern void SwigScanner_clear(SwigScanner *);
-  extern void SwigScanner_push(SwigScanner *, String *);
-  extern void SwigScanner_pushtoken(SwigScanner *, int);
-  extern int SwigScanner_token(SwigScanner *);
-  extern String *SwigScanner_text(SwigScanner *);
-  extern void SwigScanner_skip_line(SwigScanner *);
-  extern int SwigScanner_skip_balanced(SwigScanner *, int startchar, int endchar);
-  extern void SwigScanner_set_location(SwigScanner *, String *file, int line);
-  extern String *SwigScanner_get_file(SwigScanner *);
-  extern int SwigScanner_get_line(SwigScanner *);
-  extern void SwigScanner_idstart(SwigScanner *, char *idchar);
-
-#define   SWIG_MAXTOKENS          512
-#define   SWIG_TOKEN_LPAREN        1
-#define   SWIG_TOKEN_RPAREN        2
-#define   SWIG_TOKEN_SEMI          3
-#define   SWIG_TOKEN_COMMA         4
-#define   SWIG_TOKEN_STAR          5
-#define   SWIG_TOKEN_LBRACE        6
-#define   SWIG_TOKEN_RBRACE        7
-#define   SWIG_TOKEN_EQUAL         8
-#define   SWIG_TOKEN_EQUALTO       9
-#define   SWIG_TOKEN_NOTEQUAL     10
-#define   SWIG_TOKEN_PLUS         11
-#define   SWIG_TOKEN_MINUS        12
-#define   SWIG_TOKEN_AND          13
-#define   SWIG_TOKEN_LAND         14
-#define   SWIG_TOKEN_OR           15
-#define   SWIG_TOKEN_LOR          16
-#define   SWIG_TOKEN_XOR          17
-#define   SWIG_TOKEN_LESSTHAN     18
-#define   SWIG_TOKEN_GREATERTHAN  19
-#define   SWIG_TOKEN_LTEQUAL      20
-#define   SWIG_TOKEN_GTEQUAL      21
-#define   SWIG_TOKEN_NOT          22
-#define   SWIG_TOKEN_LNOT         23
-#define   SWIG_TOKEN_LBRACKET     24
-#define   SWIG_TOKEN_RBRACKET     25
-#define   SWIG_TOKEN_SLASH        26
-#define   SWIG_TOKEN_BACKSLASH    27
-#define   SWIG_TOKEN_ENDLINE      28
-#define   SWIG_TOKEN_STRING       29
-#define   SWIG_TOKEN_POUND        30
-#define   SWIG_TOKEN_PERCENT      31
-#define   SWIG_TOKEN_COLON        32
-#define   SWIG_TOKEN_DCOLON       33
-#define   SWIG_TOKEN_LSHIFT       34
-#define   SWIG_TOKEN_RSHIFT       35
-#define   SWIG_TOKEN_ID           36
-#define   SWIG_TOKEN_FLOAT        37
-#define   SWIG_TOKEN_DOUBLE       38
-#define   SWIG_TOKEN_INT          39
-#define   SWIG_TOKEN_UINT         40
-#define   SWIG_TOKEN_LONG         41
-#define   SWIG_TOKEN_ULONG        42
-#define   SWIG_TOKEN_CHAR         43
-#define   SWIG_TOKEN_PERIOD       44
-#define   SWIG_TOKEN_AT           45
-#define   SWIG_TOKEN_DOLLAR       46
-#define   SWIG_TOKEN_CODEBLOCK    47
-#define   SWIG_TOKEN_RSTRING      48
-#define   SWIG_TOKEN_LONGLONG     49
-#define   SWIG_TOKEN_ULONGLONG    50
-#define   SWIG_TOKEN_ILLEGAL      98
-#define   SWIG_TOKEN_LAST         99
+#include "swigscan.h"
 
 /* --- Functions for manipulating the string-based type encoding --- */
 
@@ -211,6 +111,7 @@ extern "C" {
   extern SwigType *SwigType_del_element(SwigType *t);
   extern SwigType *SwigType_add_pointer(SwigType *t);
   extern SwigType *SwigType_add_memberpointer(SwigType *t, const String_or_char *qual);
+  extern SwigType *SwigType_del_memberpointer(SwigType *t);
   extern SwigType *SwigType_del_pointer(SwigType *t);
   extern SwigType *SwigType_add_array(SwigType *t, const String_or_char *size);
   extern SwigType *SwigType_del_array(SwigType *t);
@@ -236,10 +137,12 @@ extern "C" {
   extern SwigType *SwigType_ltype(SwigType *t);
   extern int SwigType_ispointer(SwigType *t);
   extern int SwigType_ispointer_return(SwigType *t);
+  extern int SwigType_isfunctionpointer(SwigType *t);
   extern int SwigType_ismemberpointer(SwigType *t);
   extern int SwigType_isreference(SwigType *t);
   extern int SwigType_isreference_return(SwigType *t);
   extern int SwigType_isarray(SwigType *t);
+  extern int SwigType_prefix_is_simple_1D_array(SwigType *t);
   extern int SwigType_isfunction(SwigType *t);
   extern int SwigType_isqualifier(SwigType *t);
   extern int SwigType_isconst(SwigType *t);
@@ -250,6 +153,7 @@ extern "C" {
   extern int SwigType_isenum(SwigType *t);
   extern int SwigType_check_decl(SwigType *t, const String_or_char *decl);
   extern SwigType *SwigType_strip_qualifiers(SwigType *t);
+  extern SwigType *SwigType_functionpointer_decompose(SwigType *t);
   extern String *SwigType_base(const SwigType *t);
   extern String *SwigType_namestr(const SwigType *t);
   extern String *SwigType_templateprefix(const SwigType *t);
@@ -272,7 +176,7 @@ extern "C" {
   extern int SwigType_typedef(SwigType *type, String_or_char *name);
   extern int SwigType_typedef_class(String_or_char *name);
   extern int SwigType_typedef_using(String_or_char *qname);
-  extern void SwigType_inherit(String *subclass, String *baseclass, String *cast);
+  extern void SwigType_inherit(String *subclass, String *baseclass, String *cast, String *conversioncode);
   extern int SwigType_issubtype(SwigType *subtype, SwigType *basetype);
   extern void SwigType_scope_alias(String *aliasname, Typetab *t);
   extern void SwigType_using_scope(Typetab *t);
@@ -321,88 +225,24 @@ extern "C" {
   extern String *Swig_symbol_string_qualify(String *s, Symtab *tab);
   extern SwigType *Swig_symbol_typedef_reduce(SwigType *ty, Symtab *tab);
 
-  extern void Swig_symbol_template_defargs(Parm *parms, Parm *targs, Symtab *tscope, Symtab *tsdecl);
+  extern ParmList *Swig_symbol_template_defargs(Parm *parms, Parm *targs, Symtab *tscope, Symtab *tsdecl);
   extern SwigType *Swig_symbol_template_deftype(const SwigType *type, Symtab *tscope);
   extern SwigType *Swig_symbol_template_param_eval(const SwigType *p, Symtab *symtab);
 
 /* --- Parameters and Parameter Lists --- */
 
-/* Parameters are really just hidden behind a DOH object.  The following
-   interface will probably be simplified even further. */
+#include "swigparm.h"
 
-  extern Parm *NewParm(SwigType *type, const String_or_char *name);
-  extern Parm *NewParmFromNode(SwigType *type, const String_or_char *name, Node *n);
-  extern Parm *CopyParm(Parm *p);
-  extern ParmList *CopyParmList(ParmList *);
-  extern int ParmList_len(ParmList *);
-  extern int ParmList_numarg(ParmList *);
-  extern int ParmList_numrequired(ParmList *);
-  extern String *ParmList_str(ParmList *);
-  extern String *ParmList_str_defaultargs(ParmList *);
-  extern String *ParmList_protostr(ParmList *);
-  extern int ParmList_is_compactdefargs(ParmList *p);
-  extern int ParmList_has_defaultargs(ParmList *p);
-  extern ParmList *ParmList_copy_all_except_last_parm(ParmList *p);
+extern String    *ParmList_errorstr(ParmList *);
+extern int        ParmList_is_compactdefargs(ParmList *p);
 
 /* --- Parse tree support --- */
 
-/* DOM-like node access */
-#include "swigkeys.h"
-#define  nodeType(x)               Getattr(x,k_nodetype)
-#define  parentNode(x)             Getattr(x,k_parentnode)
-#define  previousSibling(x)        Getattr(x,k_previoussibling)
-#define  nextSibling(x)            Getattr(x,k_nextsibling)
-#define  firstChild(x)             Getattr(x,k_firstchild)
-#define  lastChild(x)              Getattr(x,k_lastchild)
-  extern int checkAttribute(Node *obj, const String_or_char *name, const String_or_char *value);
-
-/* Macros to set up the DOM tree (mostly used by the parser) */
-
-#define  set_nodeType(x,v)         Setattr(x,"nodeType",v)
-#define  set_parentNode(x,v)       Setattr(x,"parentNode",v)
-#define  set_previousSibling(x,v)  Setattr(x,"previousSibling",v)
-#define  set_nextSibling(x,v)      Setattr(x,"nextSibling",v)
-#define  set_firstChild(x,v)       Setattr(x,"firstChild",v)
-#define  set_lastChild(x,v)        Setattr(x,"lastChild",v)
-
-  extern void appendChild(Node *node, Node *child);
-  extern void preppendChild(Node *node, Node *child);
-  extern void deleteNode(Node *node);
-  extern Node *copyNode(Node *node);
-
-  extern void Swig_tag_nodes(Node *node, const String_or_char *attrname, DOH *value);
-
-  extern int Swig_require(const char *ns, Node *node, ...);
-  extern int Swig_save(const char *ns, Node *node, ...);
-  extern void Swig_restore(Node *node);
-
-/* Debugging of parse trees */
-  extern void Swig_print_tags(File *obj, Node *root);
-  extern void Swig_print_tree(Node *obj);
-  extern void Swig_print_node(Node *obj);
-
-  extern void Swig_print_xml(Node *obj, String *filename);
+#include "swigtree.h"
 
 /* -- Wrapper function Object */
 
-  typedef struct {
-    Hash *localh;
-    String *def;
-    String *locals;
-    String *code;
-  } Wrapper;
-
-  extern Wrapper *NewWrapper();
-  extern void DelWrapper(Wrapper *w);
-  extern void Wrapper_compact_print_mode_set(int flag);
-  extern void Wrapper_pretty_print(String *str, File *f);
-  extern void Wrapper_compact_print(String *str, File *f);
-  extern void Wrapper_print(Wrapper *w, File *f);
-  extern int Wrapper_add_local(Wrapper *w, const String_or_char *name, const String_or_char *decl);
-  extern int Wrapper_add_localv(Wrapper *w, const String_or_char *name, ...);
-  extern int Wrapper_check_local(Wrapper *w, const String_or_char *name);
-  extern char *Wrapper_new_local(Wrapper *w, const String_or_char *name, const String_or_char *decl);
-  extern char *Wrapper_new_localv(Wrapper *w, const String_or_char *name, ...);
+#include "swigwrap.h"
 
 /* --- Naming functions --- */
 
@@ -429,6 +269,8 @@ extern "C" {
 
   extern String *Swig_name_make(Node *n, String *prefix, String_or_char *cname, SwigType *decl, String *oldname);
   extern String *Swig_name_warning(Node *n, String *prefix, String *name, SwigType *decl);
+  extern String *Swig_name_decl(Node *n);
+  extern String *Swig_name_fulldecl(Node *n);
 
 /* --- parameterized rename functions --- */
 
@@ -436,14 +278,14 @@ extern "C" {
   extern DOH *Swig_name_object_get(Hash *namehash, String_or_char *prefix, String_or_char *name, SwigType *decl);
   extern void Swig_name_object_inherit(Hash *namehash, String *base, String *derived);
   extern void Swig_features_get(Hash *features, String_or_char *prefix, String_or_char *name, SwigType *decl, Node *n);
-  extern void Swig_feature_set(Hash *features, const String_or_char *name, SwigType *decl, const String_or_char *featurename, String *value,
-			       Hash *featureattribs);
+  extern void Swig_feature_set(Hash *features, const String_or_char *name, SwigType *decl, const String_or_char *featurename, String *value, Hash *featureattribs);
 
 /* --- Misc --- */
   extern char *Swig_copy_string(const char *c);
   extern void Swig_set_fakeversion(const char *version);
   extern const char *Swig_package_version(void);
   extern void Swig_banner(File *f);
+  extern String *Swig_strip_c_comments(const String *s);
   extern String *Swig_string_escape(String *s);
   extern String *Swig_string_mangle(const String *s);
   extern void Swig_scopename_split(String *s, String **prefix, String **last);
